@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,18 +19,31 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-u"""Store global configuration information"""
+"""Store global configuration information"""
 
 import os
-import sys
 import socket
+import sys
+import time
 
 from duplicity import __version__
-
+from duplicity import gpg
 
 # The current version of duplicity
 version = __version__
 
+# The following args are set by commandline processing
+# they correspond to the args in cli_main.duplicity_commands
+count = None
+remove_time = None
+source_path = None
+source_url = None
+target_dir = None
+target_url = None
+
+# action to take
+action = None
+
 # Prefix for all files (appended before type-specific prefixes)
 file_prefix = b""
 
@@ -43,6 +56,9 @@ file_prefix_archive = b""
 # Prefix for sig files only
 file_prefix_signature = b""
 
+# Prefix for jsonstat files only
+file_prefix_jsonstat = b""
+
 # The name of the current host
 hostname = socket.gethostname()
 
@@ -64,13 +80,14 @@ current_time = None
 # contains the signatures and manifests of the relevent backup
 # collection), and for checkpoint state between volumes.
 # NOTE: this gets expanded in duplicity.commandline
-os.environ[u"XDG_CACHE_HOME"] = os.getenv(u"XDG_CACHE_HOME", os.path.expanduser(u"~/.cache"))
-archive_dir = os.path.expandvars(u"$XDG_CACHE_HOME/duplicity")
+os.environ["XDG_CACHE_HOME"] = os.getenv("XDG_CACHE_HOME", os.path.expanduser("~/.cache"))
+archive_dir = os.path.expandvars("$XDG_CACHE_HOME/duplicity")
 archive_dir_path = None
 
 # config dir for future use
-os.environ[u"XDG_CONFIG_HOME"] = os.getenv(u"XDG_CONFIG_HOME", os.path.expanduser(u"~/.config"))
-config_dir = os.path.expandvars(u"$XDG_CONFIG_HOME/duplicity")
+os.environ["XDG_CONFIG_HOME"] = os.getenv("XDG_CONFIG_HOME", os.path.expanduser("~/.config"))
+config_dir = os.path.expandvars("$XDG_CONFIG_HOME/duplicity")
+config_dir_path = None
 
 # Restores will try to bring back the state as of the following time.
 # If it is None, default to current time.
@@ -78,7 +95,7 @@ restore_time = None
 
 # If set, restore only the subdirectory or file specified, not the
 # whole root.
-restore_dir = None
+restore_path = None
 
 # The backend representing the remote side
 backend = None
@@ -87,27 +104,29 @@ backend = None
 # See example of use in multibackend.py _list()
 # Do not use in normal cases!
 are_errors_fatal = {
-    u'delete': (True, None),
-    u'get': (True, None),
-    u'list': (True, None),
-    u'move': (True, None),
-    u'put': (True, None),
-    u'query': (True, None),
+    'delete': (True, None),
+    'get': (True, None),
+    'list': (True, None),
+    'move': (True, None),
+    'put': (True, None),
+    'query': (True, None),
 }
 
-# If set, the Select object which iterates paths in the local
-# source directory.
+# Select object which iterates paths in the local source dir.
 select = None
+select_opts = []
+select_files = []
 
+# gpg binary to use
 gpg_binary = None
 
+# Options to pass to gpg
+gpg_options = ""
+
 # Set to GPGProfile that will be used to compress/uncompress encrypted
 # files.  Replaces encryption_keys, sign_key, and passphrase settings.
 gpg_profile = None
 
-# Options to pass to gpg
-gpg_options = u''
-
 # Maximum file blocksize
 max_blocksize = 2048
 
@@ -124,62 +143,56 @@ pydevd = False
 # Character used like the ":" in time strings like
 # 2002-08-06T04:22:00-07:00.  The colon isn't good for filenames on
 # windows machines.
-time_separator = u":"
+time_separator = ":"
 
 # Global lockfile used to manage concurrency
-lockpath = u""
 lockfile = None
+lockpath = ""
 
 # If this is true, only warn and don't raise fatal error when backup
 # source directory doesn't match previous backup source directory.
-allow_source_mismatch = None
-
-# If set, abort if cannot do an incremental backup.  Otherwise if
-# signatures not found, default to full.
-incremental = None
+allow_source_mismatch = False
 
 # If set, print the statistics after every backup session
 print_statistics = True
 
-# If set, use short (< 30 char) filenames for all the remote files.
-short_filenames = False
+# If set, write extra statistic in json format to file
+jsonstat = False
 
 # If set, forces a full backup if the last full backup is older than
 # the time specified
-full_force_time = None
+full_if_older_than = None
+
+# If set the incremental backup will be skipped if not DeltaEntries are detected
+skip_if_no_change = False
+# Track if incremental backup was skipped, no cli option
+skipped_inc = False
 
 # Used to confirm certain destructive operations like deleting old files.
 force = None
 
-# If set, signifies time in seconds before which backup files should
-# be deleted.
-remove_time = None
-
 # If set, signifies the number of backups chains to keep when performing
 # a remove-all-but-n-full.
 keep_chains = None
 
-# If set, signifies that remove-all-but-n-full in progress
-remove_all_but_n_full_mode = None
-
-# If set, signifies that remove-all-inc-of-but-n-full in progress (variant of remove-all-but-n-full)
-remove_all_inc_of_but_n_full_mode = None
-
 # Don't actually do anything, but still report what would be done
 dry_run = False
 
-# If set to false, then do not encrypt files on remote system
-encryption = True
-
-# If set to false, then do not compress files on remote system
+# Compress files on remote system?
 compression = True
 
+# Encrypt files on remote system?
+encryption = True
+
 # volume size. default 200M
 volsize = 200 * 1024 * 1024
 
+# file copy blocksize
+copy_blocksize = 128 * 1024
+
 # after this volume, we will switch to multipart upload
 mp_factor = 1.1
-mp_segment_size = mp_factor * volsize
+mp_segment_size = int(mp_factor * volsize)
 
 # Working directory for the tempfile module. Defaults to /tmp on most systems.
 temproot = None
@@ -188,30 +201,21 @@ temproot = None
 timeout = 30
 
 # FTP data connection type
-ftp_connection = u'passive'
+ftp_connection = 'passive'
 
 # Header options for Webdav
-webdav_headers = u""
+webdav_headers = ""
 
 # Asynchronous put/get concurrency limit
 # (default of 0 disables asynchronicity).
 async_concurrency = 0
 
-# Whether to use "new-style" subdomain addressing for S3 buckets. Such
-# use is not backwards-compatible with upper-case buckets, or buckets
-# that are otherwise not expressable in a valid hostname.
-s3_use_new_style = False
-
-# Whether to create European buckets (sorry, hard-coded to only
-# support european for now).
-s3_european_buckets = False
-
 # File owner uid keeps number from tar file. Like same option in GNU tar.
 numeric_owner = False
 
 # Do no restore the uid/gid when finished, useful if you're restoring
 # data without having root privileges or Unix users support
-do_not_restore_ownership = False
+restore_ownership = True
 
 # Whether to use plain HTTP (without SSL) to send data to S3
 # See <https://bugs.launchpad.net/duplicity/+bug/433970>.
@@ -235,22 +239,16 @@ s3_use_deep_archive = False
 # Whether to use S3 One Zone Infrequent Access Storage
 s3_use_onezone_ia = False
 
-# True if we should use boto multiprocessing version
-s3_use_multiprocessing = False
-
 # Chunk size used for S3 multipart uploads.The number of parallel uploads to
 # S3 be given by chunk size / volume size. Use this to maximize the use of
 # your bandwidth. Defaults to 25MB
-s3_multipart_chunk_size = 25 * 1024 * 1024
+s3_multipart_chunk_size = 20 * 1024 * 1024
 
 # Minimum chunk size accepted by S3
 s3_multipart_minimum_chunk_size = 5 * 1024 * 1024
 
 # Maximum number of processes to use while doing a multipart upload to S3
-s3_multipart_max_procs = None
-
-# Maximum time to wait for a part to finish when doig a multipart upload to S3
-s3_multipart_max_timeout = None
+s3_multipart_max_procs = 4
 
 # Use server side encryption in s3
 s3_use_sse = False
@@ -265,7 +263,7 @@ s3_region_name = None
 s3_endpoint_url = None
 
 # Which storage policy to use for Swift containers
-swift_storage_policy = u""
+swift_storage_policy = ""
 
 # The largest size upload supported in a single put call for azure
 azure_max_single_put_size = None
@@ -286,18 +284,15 @@ imap_full_address = False
 
 # Name of the imap folder where we want to store backups.
 # Can be changed with a command line argument.
-imap_mailbox = u"INBOX"
+imap_mailbox = "INBOX"
 
 # Sync partial metadata by default
-metadata_sync_mode = u"partial"
-
-# Whether the old filename format is in effect.
-old_filenames = False
+metadata_sync_mode = "partial"
 
 # Wheter to specify --use-agent in GnuPG options
 use_agent = False
 
-# ssh commands to use, used by ssh_pexpect (defaults to sftp, scp)
+# ssh duplicity_commands to use, used by ssh_pexpect (defaults to sftp, scp)
 scp_command = None
 sftp_command = None
 
@@ -305,10 +300,10 @@ sftp_command = None
 ssh_askpass = False
 
 # user added ssh options
-ssh_options = u""
+ssh_options = ""
 
 # default cf backend is pyrax
-cf_backend = u"pyrax"
+cf_backend = "pyrax"
 
 # default to fully deleting files in b2
 b2_hide_files = False
@@ -319,7 +314,7 @@ ssl_cacert_path = None
 ssl_no_check_certificate = False
 
 # user added rsync options
-rsync_options = u""
+rsync_options = ""
 
 # will be a Restart object if restarting
 restart = None
@@ -358,22 +353,19 @@ progress_rate = 3
 par2_redundancy = 10
 
 # Verbatim par2 other options
-par2_options = u""
+par2_options = ""
 
 # Number of par2 volumes
 par2_volumes = 1
 
-# Whether to enable gio backend
-use_gio = False
-
 # If set, log the chnages is the set instead of the normal collection status
 show_changes_in_set = None
 
 # If set, collect only the file status, not the whole root.
 file_changed = None
 
-# If set, skip collecting the files_changed list in statistics, nullifies --file-changed
-no_files_changed = False
+# If set collect the files_changed list in statistics
+files_changed = True
 
 # delay (in seconds) before next operation after failure
 backend_retry_delay = 30
@@ -382,11 +374,17 @@ backend_retry_delay = 30
 mf_purge = False
 
 # Fake root directory path for iDrived backend
-fakeroot = None
+idr_fakeroot = None
+
+# whether to check remote manifest (requires private key)
+check_remote = True
+
+# whether 'inc` is explicit or not
+# inc_explicit = True
 
 # default filesystem encoding
-# In Python 2 it seems that sys.getfilesystemencoding() will normally return
-# 'utf-8' or some other sane encoding, but will sometimes fail and return
+# It seems that sys.getfilesystemencoding() will normally return
+# 'utf-8' or some other sane encoding, but will sometimes fail and returns
 # either 'ascii' or None.  Both are bogus, so default to 'utf-8' if it does.
 fsencoding = sys.getfilesystemencoding()
-fsencoding = fsencoding if fsencoding not in [u'ascii', u'ANSI_X3.4-1968', None] else u'utf-8'
+fsencoding = fsencoding if fsencoding not in ['ascii', 'ANSI_X3.4-1968', None] else 'utf-8'
