@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,19 +19,17 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-u"""Generate and process backup statistics"""
-from __future__ import division
-
-from builtins import zip
-from builtins import map
-from builtins import str
-from builtins import object
+"""Generate and process backup statistics"""
 
+import json
+import os
 import re
 import time
-import os
 
+from duplicity import config
 from duplicity import dup_time
+from duplicity import util
+from duplicity import log
 
 
 class StatsException(Exception):
@@ -39,97 +37,90 @@ class StatsException(Exception):
 
 
 class StatsObj(object):
-    u"""Contains various statistics, provide string conversion functions"""
+    """Contains various statistics, provide string conversion functions"""
     # used when quoting files in get_stats_line
-    space_regex = re.compile(u" ")
+    space_regex = re.compile(" ")
 
-    stat_file_attrs = (u'SourceFiles',
-                       u'SourceFileSize',
-                       u'NewFiles',
-                       u'NewFileSize',
-                       u'DeletedFiles',
-                       u'ChangedFiles',
-                       u'ChangedFileSize',
-                       u'ChangedDeltaSize',
-                       u'DeltaEntries',
-                       u'RawDeltaSize')
-    stat_misc_attrs = (u'Errors',
-                       u'TotalDestinationSizeChange')
-    stat_time_attrs = (u'StartTime',
-                       u'EndTime',
-                       u'ElapsedTime')
-    stat_attrs = ((u'Filename',) + stat_time_attrs +
+    stat_file_attrs = ('SourceFiles',
+                       'SourceFileSize',
+                       'NewFiles',
+                       'NewFileSize',
+                       'DeletedFiles',
+                       'ChangedFiles',
+                       'ChangedFileSize',
+                       'ChangedDeltaSize',
+                       'DeltaEntries',
+                       'RawDeltaSize')
+    stat_misc_attrs = ('Errors',
+                       'TotalDestinationSizeChange')
+    stat_time_attrs = ('StartTime',
+                       'EndTime',
+                       'ElapsedTime')
+    stat_attrs = (('Filename',) + stat_time_attrs +
                   stat_misc_attrs + stat_file_attrs)
 
     # Below, the second value in each pair is true iff the value
     # indicates a number of bytes
-    stat_file_pairs = ((u'SourceFiles', False),
-                       (u'SourceFileSize', True),
-                       (u'NewFiles', False),
-                       (u'NewFileSize', True),
-                       (u'DeletedFiles', False),
-                       (u'ChangedFiles', False),
-                       (u'ChangedFileSize', True),
-                       (u'ChangedDeltaSize', True),
-                       (u'DeltaEntries', False),
-                       (u'RawDeltaSize', True))
+    stat_file_pairs = (('SourceFiles', False),
+                       ('SourceFileSize', True),
+                       ('NewFiles', False),
+                       ('NewFileSize', True),
+                       ('DeletedFiles', False),
+                       ('ChangedFiles', False),
+                       ('ChangedFileSize', True),
+                       ('ChangedDeltaSize', True),
+                       ('DeltaEntries', False),
+                       ('RawDeltaSize', True))
 
     # This is used in get_byte_summary_string below
-    byte_abbrev_list = ((1024 * 1024 * 1024 * 1024, u"TB"),
-                        (1024 * 1024 * 1024, u"GB"),
-                        (1024 * 1024, u"MB"),
-                        (1024, u"KB"))
+    byte_abbrev_list = ((1024 * 1024 * 1024 * 1024, "TB"),
+                        (1024 * 1024 * 1024, "GB"),
+                        (1024 * 1024, "MB"),
+                        (1024, "KB"))
 
     def __init__(self):
-        u"""Set attributes to None"""
+        """Set attributes to None"""
         for attr in self.stat_attrs:
             self.__dict__[attr] = None
 
     def get_stat(self, attribute):
-        u"""Get a statistic"""
+        """Get a statistic"""
         return self.__dict__[attribute]
 
     def set_stat(self, attr, value):
-        u"""Set attribute to given value"""
+        """Set attribute to given value"""
         self.__dict__[attr] = value
 
     def increment_stat(self, attr):
-        u"""Add 1 to value of attribute"""
+        """Add 1 to value of attribute"""
         self.__dict__[attr] += 1
 
-    def get_total_dest_size_change(self):
-        u"""Return total destination size change
-
-        This represents the total increase in the size of the
-        duplicity destination directory, or None if not available.
-
-        """
-        return 0  # this needs to be re-done for duplicity
-
     def get_stats_line(self, index, use_repr=1):
-        u"""Return one line abbreviated version of full stats string"""
+        """Return one line abbreviated version of full stats string"""
         file_attrs = [str(self.get_stat(a)) for a in self.stat_file_attrs]
         if not index:
-            filename = u"."
+            filename = "."
         else:
             filename = os.path.join(*index)
             if use_repr:
                 # use repr to quote newlines in relative filename, then
                 # take of leading and trailing quote and quote spaces.
-                filename = self.space_regex.sub(u"\\\\x20", repr(filename))
+                filename = self.space_regex.sub("\\\\x20", repr(filename))
                 n = 1
-                if filename[0] == u'u':
+                if filename[0] == 'u':
                     n = 2
                 filename = filename[n:-1]
-        return u" ".join([filename, ] + file_attrs)
+        return " ".join([filename, ] + file_attrs)
 
     def set_stats_from_line(self, line):
-        u"""Set statistics from given line"""
+        """Set statistics from given line"""
+
         def error():
-            raise StatsException(u"Bad line '%s'" % line)
-        if line[-1] == u"\n":
+            raise StatsException(f"Bad line '{line}'")
+
+        if line[-1] == "\n":
             line = line[:-1]
-        lineparts = line.split(u" ")
+        lineparts = line.split(" ")
         if len(lineparts) < len(self.stat_file_attrs):
             error()
         for attr, val_string in zip(self.stat_file_attrs,
@@ -145,62 +136,58 @@ class StatsObj(object):
         return self
 
     def get_stats_string(self):
-        u"""Return extended string printing out statistics"""
-        return u"%s%s%s" % (self.get_timestats_string(),
-                            self.get_filestats_string(),
-                            self.get_miscstats_string())
+        """Return extended string printing out statistics"""
+        return f"{self.get_timestats_string()}{self.get_filestats_string()}{self.get_miscstats_string()}"
 
     def get_timestats_string(self):
-        u"""Return portion of statistics string dealing with time"""
+        """Return portion of statistics string dealing with time"""
         timelist = []
         if self.StartTime is not None:
-            timelist.append(u"StartTime %.2f (%s)\n" %  # pylint: disable=bad-string-format-type
+            timelist.append("StartTime %.2f (%s)\n" %  # pylint: disable=bad-string-format-type
                             (self.StartTime, dup_time.timetopretty(self.StartTime)))
         if self.EndTime is not None:
-            timelist.append(u"EndTime %.2f (%s)\n" %  # pylint: disable=bad-string-format-type
+            timelist.append("EndTime %.2f (%s)\n" %  # pylint: disable=bad-string-format-type
                             (self.EndTime, dup_time.timetopretty(self.EndTime)))
-        if self.ElapsedTime or (self.StartTime is not None and
+        if self.ElapsedTime or (self.StartTime is not None and  # pylint:disable=access-member-before-definition
                                 self.EndTime is not None):
-            if self.ElapsedTime is None:
+            if self.ElapsedTime is None:  # pylint:disable=access-member-before-definition
                 self.ElapsedTime = self.EndTime - self.StartTime
-            timelist.append(u"ElapsedTime %.2f (%s)\n" %
-                            (self.ElapsedTime, dup_time.inttopretty(self.ElapsedTime)))
-        return u"".join(timelist)
+            timelist.append(f"ElapsedTime {self.ElapsedTime:.2f} ({dup_time.inttopretty(self.ElapsedTime)})\n")
+        return "".join(timelist)
 
     def get_filestats_string(self):
-        u"""Return portion of statistics string about files and bytes"""
+        """Return portion of statistics string about files and bytes"""
+
         def fileline(stat_file_pair):
-            u"""Return zero or one line of the string"""
+            """Return zero or one line of the string"""
             attr, in_bytes = stat_file_pair
             val = self.get_stat(attr)
             if val is None:
-                return u""
+                return ""
             if in_bytes:
-                return u"%s %s (%s)\n" % (attr, val,
-                                          self.get_byte_summary_string(val))
+                return f"{attr} {val} ({self.get_byte_summary_string(val)})\n"
             else:
-                return u"%s %s\n" % (attr, val)
+                return f"{attr} {val}\n"
 
-        return u"".join(map(fileline, self.stat_file_pairs))
+        return "".join(map(fileline, self.stat_file_pairs))
 
     def get_miscstats_string(self):
-        u"""Return portion of extended stat string about misc attributes"""
-        misc_string = u""
+        """Return portion of extended stat string about misc attributes"""
+        misc_string = ""
         tdsc = self.TotalDestinationSizeChange
         if tdsc is not None:
-            misc_string += (u"TotalDestinationSizeChange %s (%s)\n" %
-                            (tdsc, self.get_byte_summary_string(tdsc)))
+            misc_string += f"TotalDestinationSizeChange {tdsc} ({self.get_byte_summary_string(tdsc)})\n"
         if self.Errors is not None:
-            misc_string += u"Errors %d\n" % self.Errors
+            misc_string += f"Errors {int(self.Errors)}\n"
         return misc_string
 
     def get_byte_summary_string(self, byte_count):
-        u"""Turn byte count into human readable string like "7.23GB" """
+        """Turn byte count into human readable string like "7.23GB" """
         if byte_count < 0:
-            sign = u"-"
+            sign = "-"
             byte_count = -byte_count
         else:
-            sign = u""
+            sign = ""
 
         for abbrev_bytes, abbrev_string in self.byte_abbrev_list:
             if byte_count >= abbrev_bytes:
@@ -212,26 +199,89 @@ class StatsObj(object):
                     precision = 1
                 else:
                     precision = 2
-                return u"%s%%.%df %s" % (sign, precision, abbrev_string) \
-                       % (abbrev_count,)
+                return f"{sign}{abbrev_count:.{precision}f} {abbrev_string}"
         byte_count = round(byte_count)
         if byte_count == 1:
-            return sign + u"1 byte"
+            return f"{sign}1 byte"
         else:
-            return u"%s%d bytes" % (sign, byte_count)
+            return f"{sign}{int(byte_count)} bytes"
+
+    def get_stats_json(self, col_stat):
+        """
+        Return enriched statistics in JSON format
+        @type col_stat: dup_collections.CollectionsStatus
+        @param col_stat: allow to gather information about the whole
+            backup chain
+
+        @rtype: String
+        @return: JSON formated string
+        """
+
+        def fail_save_read(parent, *attributes, is_function=False, default="N/A"):
+            """
+            retruns "N/A" if value can't de determined.
+            @type parent: object
+            @param parent: oject where vale should received from. Oject must exists
+            @type *attributes: list of str
+            @param *attribures: path down to the attribute that should be read
+            @type is_function: boolean
+            @param is_function: run last attribute as function instead of reading the value direct
+            @param default: overwrite return value if value can't be determined
+            """
+            try:
+                attr_path = parent
+                for attibute in attributes:
+                    try:
+                        attr_path = getattr(attr_path, attibute)
+                    except AttributeError:
+                        return default
+                if is_function:
+                    return attr_path()
+                else:
+                    return attr_path
+            except Exception as e:
+                log.Error(f"Can't read expected attribute: {e}")
+                return default
+
+        py_obj = {key: self.__dict__[key] for key in self.stat_attrs}
+        for t in ('StartTime', 'EndTime'):
+            t_str = f"{t}_str"
+            py_obj[t_str] = dup_time.timetostring(py_obj[t])
+        if col_stat:
+            backup_meta = {}
+            backup_chain = col_stat.matched_chain_pair[1]
+            backup_meta["action"] = col_stat.action
+            backup_meta["skipped_inc"] = config.skipped_inc
+            backup_meta["time_full_bkp"] = backup_chain.fullset.time
+            backup_meta["time_full_bkp_str"] = dup_time.timetostring(backup_meta["time_full_bkp"])
+            backup_meta["no_of_inc"] = len(backup_chain.incset_list)
+            backup_meta["target"] = fail_save_read(config, "target_url")
+            backup_meta["source"] = fail_save_read(config, "source_path")
+            backup_meta["local_json_stat"] = [fail_save_read(
+                backup_chain, "fullset", "local_jsonstat_path", "get_filename", is_function=True)]
+            for inc in backup_chain.incset_list:
+                backup_meta["local_json_stat"].append(fail_save_read(
+                    inc, "local_jsonstat_path", "get_filename", is_function=True))
+            py_obj["backup_meta"] = backup_meta
+
+        return json.dumps(
+            py_obj,
+            cls=util.BytesEncoder, indent=4
+        )
 
     def get_stats_logstring(self, title):
-        u"""Like get_stats_string, but add header and footer"""
-        header = u"--------------[ %s ]--------------" % title
-        footer = u"-" * len(header)
-        return u"%s\n%s%s\n" % (header, self.get_stats_string(), footer)
+        """Like get_stats_string, but add header and footer"""
+        header = f"--------------[ {title} ]--------------"
+        footer = "-" * len(header)
+        return f"{header}\n{self.get_stats_string()}{footer}\n"
 
     def set_stats_from_string(self, s):
-        u"""Initialize attributes from string, return self for convenience"""
-        def error(line):
-            raise StatsException(u"Bad line '%s'" % line)
+        """Initialize attributes from string, return self for convenience"""
 
-        for line in s.split(u"\n"):
+        def error(line):
+            raise StatsException(f"Bad line '{line}'")
+
+        for line in s.split("\n"):
             if not line:
                 continue
             line_parts = line.split()
@@ -255,20 +305,20 @@ class StatsObj(object):
         return self
 
     def write_stats_to_path(self, path):
-        u"""Write statistics string to given path"""
-        fin = path.open(u"w")
+        """Write statistics string to given path"""
+        fin = path.open("w")
         fin.write(self.get_stats_string())
         assert not fin.close()
 
     def read_stats_from_path(self, path):
-        u"""Set statistics from path, return self for convenience"""
-        fp = path.open(u"r")
+        """Set statistics from path, return self for convenience"""
+        fp = path.open("r")
         self.set_stats_from_string(fp.read())
         assert not fp.close()
         return self
 
     def stats_equal(self, s):
-        u"""Return true if s has same statistics as self"""
+        """Return true if s has same statistics as self"""
         assert isinstance(s, StatsObj)
         for attr in self.stat_file_attrs:
             if self.get_stat(attr) != s.get_stat(attr):
@@ -276,7 +326,7 @@ class StatsObj(object):
         return 1
 
     def set_to_average(self, statobj_list):
-        u"""Set self's attributes to average of those in statobj_list"""
+        """Set self's attributes to average of those in statobj_list"""
         for attr in self.stat_attrs:
             self.set_stat(attr, 0)
         for statobj in statobj_list:
@@ -298,7 +348,7 @@ class StatsObj(object):
         return self
 
     def get_statsobj_copy(self):
-        u"""Return new StatsObj object with same stats as self"""
+        """Return new StatsObj object with same stats as self"""
         s = StatsObj()
         for attr in self.stat_attrs:
             s.set_stat(attr, self.get_stat(attr))
@@ -306,9 +356,10 @@ class StatsObj(object):
 
 
 class StatsDeltaProcess(StatsObj):
-    u"""Keep track of statistics during DirDelta process"""
+    """Keep track of statistics during DirDelta process"""
+
     def __init__(self):
-        u"""StatsDeltaProcess initializer - zero file attributes"""
+        """StatsDeltaProcess initializer - zero file attributes"""
         StatsObj.__init__(self)
         for attr in StatsObj.stat_file_attrs:
             self.__dict__[attr] = 0
@@ -317,7 +368,7 @@ class StatsDeltaProcess(StatsObj):
         self.files_changed = []
 
     def add_new_file(self, path):
-        u"""Add stats of new file path to statistics"""
+        """Add stats of new file path to statistics"""
         filesize = path.getsize()
         self.SourceFiles += 1
         # SourceFileSize is added-to incrementally as read
@@ -327,7 +378,7 @@ class StatsDeltaProcess(StatsObj):
         self.add_delta_entries_file(path, b'new')
 
     def add_changed_file(self, path):
-        u"""Add stats of file that has changed since last backup"""
+        """Add stats of file that has changed since last backup"""
         filesize = path.getsize()
         self.SourceFiles += 1
         # SourceFileSize is added-to incrementally as read
@@ -337,23 +388,23 @@ class StatsDeltaProcess(StatsObj):
         self.add_delta_entries_file(path, b'changed')
 
     def add_deleted_file(self, path):
-        u"""Add stats of file no longer in source directory"""
+        """Add stats of file no longer in source directory"""
         self.DeletedFiles += 1  # can't add size since not available
         self.DeltaEntries += 1
         self.add_delta_entries_file(path, b'deleted')
 
     def add_unchanged_file(self, path):
-        u"""Add stats of file that hasn't changed since last backup"""
+        """Add stats of file that hasn't changed since last backup"""
         filesize = path.getsize()
         self.SourceFiles += 1
         self.SourceFileSize += filesize
 
     def close(self):
-        u"""End collection of data, set EndTime"""
+        """End collection of data, set EndTime"""
         self.EndTime = time.time()
 
     def add_delta_entries_file(self, path, action_type):
-        if path.isreg():
+        if config.files_changed and path.isreg():
             self.files_changed.append((path.get_relative_path(), action_type))
 
     def get_delta_entries_file(self):
